@@ -6,7 +6,8 @@ print.RV.multcomp <- function (x,digits=4,...) {
   if (exists("data.name",where=x)) {
     cat(paste("data:  ",x$data.name,"\n\n",sep=""))
   }
-  print(x$p.value,digits=digits,na.print="-",row.names=FALSE)
+  rown <- if (is.null(rownames(x))) {FALSE} else {TRUE}
+  print(x$p.value,digits=digits,na.print="-",row.names=rown)
   cat(paste("\nP value adjustment method: ",x$p.adjust.method,"\n",sep=""))
   invisible(x)
 }
