@@ -1,6 +1,6 @@
 user.cont <- function(cont) {
   user.cont.lsmc <- function(levs,...) {
-    M <- as.data.frame(cont)
+    M <- as.data.frame(t(cont))
     for (i in 1:ncol(M)) {
 	colnames(M)[i] <- paste(paste(rownames(M)[M[,i]>0],collapse="-"),
 	  paste(rownames(M)[M[,i]<0],collapse="-"),sep=" vs ")
